// Simple Multiuser Server
// (c) Copyright Justin Couch   justin@vlc.com.au
// The Virtual Light Company 1996
//
// From Chapter 20: Late Night VRML 2.0 and java
//
// This is the Chat server starter class. It fires off the connection manager
// and uses TCP connections

package chat;

import java.net.*;
import java.io.*;
import filter.UserServer;

public class Chat extends Thread
{
	private static final int BUFFER_LENGTH = 1024;
	private int	_port;
	private boolean		server_available = false;
	private DatagramSocket	listen_socket = null;
	private ThreadGroup	thread_group;
	private ChatManager	manager = null;
	private UserServer	server;
	
	public Chat(int port, ThreadGroup tg, UserServer userver)
	{
		super(tg, "Chat server thread");

		thread_group = tg;

		_port = port;

		System.out.println("Starting Chat Server on UDP port " + port);
		try
		{
			listen_socket = new DatagramSocket(port);
		}
		catch(IOException e)
		{
			System.err.println("Unable to open Chat socket " + e.getMessage());
			return;
		}

		manager = new ChatManager();

		server = userver;
		server_available = true;
	}

	public void run()
	{
		byte[]	buffer;
		DatagramPacket packet;
		ChatMessage a_chat;

		// if the server failed to start then exit and do nothing.
		if(!server_available)
			return;

		// tell the chat manager to start running
		manager.start();

		// now process the input.
		while(true)
		{
			buffer = new byte[BUFFER_LENGTH];
			packet = new DatagramPacket(buffer, BUFFER_LENGTH);

			try
			{
				listen_socket.receive(packet);
				Thread.yield();
			}
			catch(IOException e)
			{
				System.err.println("packet error");
			}

			a_chat = new ChatMessage(packet, thread_group, server);
			a_chat.run();
			manager.add(a_chat);
		}
	}
}
